// ************** KNiXuino - KNX Bus and Arduino ****************
// for further info check http://www.knixuino.com
// Version info - _readme.txt
// Copyright info - _licence.txt
// Do not remove this headerinfo !!!!


#ifndef KNXBasicObjectTypes_h
#define KNXBasicObjectTypes_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif
  
#include "KNXConnection.h" 

 
// KNXSwitch managing any boolean value. Very often a on/off switch. 
// Also a Status Group Address can be defined, where the Actor can send 
// Status Messages back.
class KNXBit:public KNXObject {

	public:
		KNXBit():KNXObject() { }
		KNXBit(uint16_t ga): KNXObject(ga) { }
		KNXBit(uint16_t ga, uint16_t status): KNXObject(ga, status) { }


		// KNXBit();
		// receivedByKNX is used for communication with the KNXCommunication Object. 
		// When a value on a group address is received, that the Object is 
		// listening to, than the value is sent to the object via that method.
		// To set the value within the arduino code. See methods below. 
	    virtual void receivedFromKNX(uint16_t ga, uint8_t value);
		virtual void print();
		 		
		// outgoing methods
		// use these methods within arduino code to write out to knx
		virtual void on() { set(1); }
		virtual void off() { set(0); }
		virtual void toggle() { set(!get()); }
		virtual void set(uint8_t);
		virtual uint8_t get() { return getDataBit(); };
	

		// bit methods - for accessing data bits of flags (1= bit data)
		void setDataBit(boolean value) {if (value) bitSet(flags, 1); else bitClear(flags, 1);}
		uint8_t getDataBit() {return bitRead(flags, 1);}
};





 
class KNXByte:public KNXObject {
	public:
		KNXByte(): KNXObject() { }
		KNXByte(uint16_t ga): KNXObject(ga) { }
		KNXByte(uint16_t ga, uint16_t status): KNXObject(ga, status) { }

	    virtual void receivedFromKNX(uint16_t ga, uint8_t value);
		virtual void print();
		 		
		// outgoing methods
		// use these methods within arduino code to write out to knx
		virtual void set(uint8_t);
		virtual uint8_t get() { return data; };
	
	private:
		uint8_t data;
	
};





#endif //KNXBasicObjectTypes_h