// ************** KNiXuino - KNX Bus and Arduino ****************
// for further info check http://www.knixuino.com
// Version info - _readme.txt
// Copyright info - _licence.txt
// Do not remove this headerinfo !!!!


#ifndef KNXConnection_h
#define KNXConnection_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include <elapsedMillis.h>

// comment or define the port where debug messages should be print.
#define KNXConnectionDebug Serial
#define MaxListeningObjects 256


// group address functions. (a group address is stroed in one 16 bit variable.)
// conversion between 3-part group address and 16bit value is done with these functions

    uint16_t gaToInt (uint8_t first, uint8_t second, uint8_t third);
    uint8_t getFirst(uint16_t ad);
    uint8_t getSecond(uint16_t ad);    
    uint8_t getThird(uint16_t ad);
        


// abstract class for any KNX Object. You have to implement these methods for communication with the KNXConnection class. 
class KNXObject
{
	public:
	KNXObject();
	KNXObject(uint16_t ga);
	KNXObject(uint16_t ga, uint16_t status);

		
	// Group Addresses 
	void setGroupAddress(uint16_t);
	uint16_t getGroupAddress() { return groupAddress; };
	virtual uint8_t isCheckingStatus() { return statusGA!=NULL;} // returns if the object has any status GA
	void setStatusGA(uint16_t);
	uint16_t getStatusGroupAddress() { return statusGA; };
	virtual boolean isListeningTo(uint16_t ga);

	// bit methods - for accessing data bits (see below)
	void setStatusBit(boolean value) {if (value) bitSet(flags, 2); else bitClear(flags, 2);}
	uint8_t getStatusBit() {return bitRead(flags, 2);}

	void setInternalBit(boolean value) {if (value) bitSet(flags, 3); else bitClear(flags, 3);}
	uint8_t getInternalBit() {return bitRead(flags, 3);}

	void setWebOutBit(boolean value) {if (value) bitSet(flags, 4); else bitClear(flags, 4);}
	uint8_t getWebOutBit() {return bitRead(flags, 4);}

	void setWebInBit(boolean value) {if (value) bitSet(flags, 5); else bitClear(flags, 5);}
	uint8_t getWebInBit() {return bitRead(flags, 5);}

	void setWebBits(boolean value) {setWebOutBit(value); setWebInBit(value);}

	void resetLastChange() { this->lastChange = 0; }
	elapsedMillis getLastChange() { return lastChange; }

	// value was received from KNX Bus.
    virtual void receivedFromKNX(uint16_t ga, uint8_t value);
    virtual void print() {Serial.println("Method print not implemented for this Class.");}

    // set the value of the object
    virtual void set(uint8_t data) {}
    virtual uint8_t get() {}

	protected:
		uint8_t flags; // flags !
		// bit 1: left blank (used by bit object as data bit)
		// bit 2: status (Status confirmed)
		// bit 3: internal (Arduino is the Device itself)
		//      internal = there is no external device. arduino is the device itself. so after
		//      receiving a new value a status message is sent back if the statusGA is defined
		// bit 4: Web Out -> Values are sent to the WEB (= can be read in the WebApp)
		// bit 5: Web In  -> Values may be changed via the web.

		uint16_t groupAddress; // main GA for swicthing ob/off
		uint16_t statusGA; // GA for incoming Status Messages (usually resonse from actor)
		elapsedMillis lastChange; // millis since last change of value

};



// abstract class for an ETS conection class. Implement these methods to connect to KNiXuino 
class KNXConnection
{
	public:
		KNXConnection();
	// write out 8-bit integer data to KNX
		virtual void write(uint16_t ga, uint8_t data, uint8_t bits) {};

	// sends the received value to all listening objects
		virtual void sendDataToObject(uint16_t ga, uint8_t data);

	// method called within the loop -> all loop functions come here !
		virtual void doIt() {};


	// listening Objects / Object management
		virtual void addListeningObject(KNXObject*);
		virtual KNXObject* nextObject() { return listeningObjects[index++]; }
		
		
		virtual bool isLast() { return ((listeningObjects[index+1] != 0) && (listeningObjects[index+1]!=NULL)); }
		virtual void restartIterator() { index=0; }

		
		KNXObject* getObjectFromGA(uint16_t ga);
		

	private:
		KNXObject *listeningObjects[MaxListeningObjects];
		uint8_t index; // current index of array (for iterator)

};



#endif 