// ************** KNiXuino - KNX Bus and Arduino ****************
// for further info check http://www.knixuino.com
// Version info - _readme.txt
// Copyright info - _licence.txt
// Do not remove this headerinfo !!!!


#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include "KNXConnection_SIMKNX_raw.h"


KNXConnection_SIMKNX_raw::KNXConnection_SIMKNX_raw() {
	KNXConnection::KNXConnection();
	term='\r'; // terminating character of a knx message
	KNXSerialStream = &Serial1;
	KNXSerialStream->begin(9600);

	clearBuffer(); 
}


void KNXConnection_SIMKNX_raw::write(uint16_t ga, uint8_t data, uint8_t bits) {
	KNXSerialStream->print("tds ($");
	KNXSerialStream->print(ga, HEX);
	if (bits < 8) KNXSerialStream->print(") ");
	else KNXSerialStream->print(" $01) ");
	KNXSerialStream->println(data);
	}




//
// Initialize the command buffer being processed to all null characters
//
void KNXConnection_SIMKNX_raw::clearBuffer()
{
	for (int i = 0; i < KNXStreamBUFFER; i++) {
		buffer[i]='\0';
	}
	bufPos=0; 
}



// -- Stream auf eingegenagen Daten prüfen --
void KNXConnection_SIMKNX_raw::readSerial() 
{
	while (KNXSerialStream->available() > 0) 
	{
		int i; 
		incomingByte= KNXSerialStream->read();  				
		if (incomingByte==term) {    		
			bufPos=0;       
			#if defined(KNXConnectionDebug)
				KNXConnectionDebug.print("Read following KNX Stream:");
				KNXConnectionDebug.println(getBuffer());
			#endif
   
			incoming(); 
			clearBuffer(); 
		}
		else    
		{
			if (isprint(incomingByte)) {
				buffer[bufPos++]=incomingByte; 
				buffer[bufPos]='\0';  
				if (bufPos > KNXStreamBUFFER-1) bufPos=0; 
			}
		}
	}
}



void KNXConnection_SIMKNX_raw::incoming() {
	char *tok;
	char *limits;
	limits=" <>()$";
	
	Serial.println(getBuffer());
	
    tok = strtok_r(getBuffer(), limits, &last);
	
	if (strcmp("tdi", tok)==0 || strcmp("tdc", tok)==0) {
		// tdi (source dest length) data
		// source = physical address of sending device
		// dest = group address
		// length = length in bytes 0 = 1-6 bit 
		// data = data (hex)
		// tdi ($111E $1924 $00) $01
		// tdc is sent as confirmation from the sim-KNX if data is written to KNX Bus via SIM-KNX
		// tdc is needed if Data is sent to KNX via WebApp, then the TDC Message also updates the 
		// KNX Objects if present.
		char part[8];
		strcpy (part,"0x");
		strcat(part, strtok_r(NULL, limits, &last));
		int pa = strtol(part, NULL, 16);
		int ga = int(strtol(strtok_r(NULL, limits, &last), NULL, 16));
		int length = int(strtol(strtok_r(NULL, limits, &last), NULL, 16));
				
		int dat = int(strtol(strtok_r(NULL, limits, &last), NULL, 16));
		#if defined(KNXConnectionDebug)
			//KNXConnectionDebug.print("SUCCESS: Value parsed: ");
			//KNXConnectionDebug.println(dat);
		#endif
		sendDataToObject(ga, dat);
	}	
	




	

	
	
} // end method incoming


