// ************** KNiXuino - KNX Bus and Arduino ****************
// for further info check http://www.knixuino.com
// Version info - _readme.txt
// Copyright info - _licence.txt
// Do not remove this headerinfo !!!!


#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include "KNXPresenceSimulation.h"


KNXPresenceSimulation::KNXPresenceSimulation() {
	objectList = NULL;
	// Presence Simulation is a "switch" that is handled inside the Arduino
	// so it is internal. That makes it switchable via the KNX when a groupAddress
	// is set.
	setInternalBit(1);
}


void KNXPresenceSimulation::doIt() {
	if (get() == 0) return; // Presence SImulation is switched off
	KNXPresenceSimulationObj *ptr = objectList;
	while(ptr != NULL) {
		ptr->doIt();
		ptr = ptr->next;		
	}
} // end method doIt

void KNXPresenceSimulation::addObject(KNXPresenceSimulationObj* obj){
	obj->next = objectList;
	objectList = obj;	
}

void KNXPresenceSimulation::set(uint8_t d) {
			#if defined(KNXConnectionDebug)
				KNXConnectionDebug.print("Presence Simulator switched to:");
				KNXConnectionDebug.println(d);
			#endif

	KNXBit::set(d);
	if (!d) {
		allOff();
		}
}


void KNXPresenceSimulation::allOff() {
	KNXPresenceSimulationObj *ptr = objectList;
	while(ptr != NULL) {
		ptr->off();
		ptr = ptr->next;		
	}

}



void KNXPresenceSimulationObj::off() {
	object->off();
	}



KNXPresenceSimulationObj::KNXPresenceSimulationObj(KNXBit *obj, int minOn, int maxOn, int minOff, int maxOff)
{
	object = obj;
	minOnTime = minOn;
	minOffTime = minOff;
	maxOnTime = maxOn;
	maxOffTime = maxOff;
	actualValue = 0;
	next = NULL;
	elapsed = 0;
	
	planNextChange();
}


void KNXPresenceSimulationObj::doIt() {
	if ((elapsed/1000) > delay) {
		toggle();
		planNextChange();	
	}
}



void KNXPresenceSimulationObj::toggle() {
	actualValue = !actualValue;
	object->set(actualValue);
}



void KNXPresenceSimulationObj::planNextChange() {
	elapsed = 0;
	if (actualValue) {
		delay = random(minOnTime, maxOnTime);
	}	
	else
	{
		delay = random(minOffTime, maxOffTime);
	}	
			#if defined(KNXConnectionDebug)
				KNXConnectionDebug.print("Presence Simulator: next toggle in sec:");
				KNXConnectionDebug.println(delay);
			#endif
}

